@@ -5,7 +5,6 @@ import numpy as np
 import ray
 
 from plangym.core import BaseEnvironment, VectorizedEnvironment
-from plangym.parallel import batch_step_data
 
 
 @ray.remote
@@ -171,15 +170,12 @@ class RayEnv(VectorizedEnvironment):
             observs, rewards, dones, infos = self._make_transitions(actions, None, dt)
         else:
             states, observs, rewards, dones, infos = self._make_transitions(actions, states, dt)
-        try:
-            observ = np.stack(observs)
-            reward = np.stack(rewards)
-            done = np.stack(dones)
-            infos = np.stack(infos)
-        except BaseException as e:  # Lets be overconfident for once TODO: remove this.
-            print(e)
-            for obs in observs:
-                print(obs.shape)
+
+        observ = np.stack(observs)
+        reward = np.stack(rewards)
+        done = np.stack(dones)
+        infos = np.stack(infos)
+
         if states is None:
             return observ, reward, done, infos
         else:
@@ -211,7 +207,7 @@ class RayEnv(VectorizedEnvironment):
 
     def _make_transitions(self, actions, states=None, dt: [np.ndarray, int] = 1):
         no_states = states is None or states[0] is None
-        states_chunks, actions_chunks, dt_chunks = batch_step_data(
+        states_chunks, actions_chunks, dt_chunks = self.batch_step_data(
             actions=actions,
             states=states,
             dt=dt,
