@@ -25,13 +25,14 @@ def ray_dm_control():
     return RayEnv(env_class=DMControlEnv, name="walker-walk", n_workers=2)
 
 
-environments = [ray_cartpole, ray_retro, ray_dm_control]
-
-with warnings.catch_warnings():
-    warnings.simplefilter("ignore")
-    ray.init(ignore_reinit_error=True)
+environments = [(ray_cartpole, True), (ray_retro, False), (ray_dm_control, True)]
 
 
 @pytest.fixture(params=environments, scope="class")
 def env(request) -> AtariEnvironment:
-    return request.param()
+    env_call, local = request.param
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        ray.init(ignore_reinit_error=True, local_mode=local)
+    yield env_call()
+    ray.shutdown()
